<?php

namespace Drupal\paywall_simplenews_subscription\Plugin\RolePaywallAccessRule;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\role_paywall\Plugin\RolePaywallAccessRuleBase;
use Drupal\simplenews\Entity\Subscriber;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a user role access rule for the Role Paywall.
 *
 * @RolePaywallAccessRule(
 *   id = "simplenews_subscription",
 *   label = @Translation("Simplenews subscription"),
 *   description = @Translation("Evaluate access based on a simplenews_subscription"),
 * )
 */
class RolePaywallSimplenewsSubscription extends RolePaywallAccessRuleBase implements ContainerFactoryPluginInterface {

  /**
   * Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a Permission object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
          $configuration,
          $plugin_id,
          $plugin_definition,
          $container->get('entity_type.manager'),
      );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    // The default configuration will be have the block hidden (0).
    return ['required_newsletters' => ''] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function checkAccess(AccountInterface $account, $entity_type, ContentEntityInterface $entity) {
    $subscriber = Subscriber::loadByUid($account->id(), FALSE, TRUE);
    foreach ($this->configuration['required_newsletters'] as $required_newsletter) {
      if ($subscriber && $subscriber->isSubscribed($required_newsletter)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm($form, FormStateInterface $form_state) {
    parent::buildConfigurationForm($form, $form_state);
    // Get list of newsletters.
    $site_newsletters = simplenews_newsletter_list();

    $form['required_newsletters'] = [
      '#type' => 'checkboxes',
      '#options' => $site_newsletters,
      '#title' => $this->t('Required newsletter subscription'),
      '#required' => TRUE,
      '#default_value' => empty($this->configuration['required_newsletters']) ? [] : $this->configuration['required_newsletters'],
      '#description' => $this->t('Users subscribed to the selected newsletters will be able to access the content.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $required_newsletters = array_filter($form_state->getValue('required_newsletters'));
    $this->configuration['required_newsletters'] = array_keys($required_newsletters);
  }

  /**
   * {@inheritdoc}
   */
  public function getConfigurationSummary() {
    if (empty($this->configuration['required_newsletters'])) {
      return $this->t('No newsletters selected');
    }
    else {
      $required_newsletters = $this->configuration['required_newsletters'];
      $newsletters = $this->entityTypeManager->getStorage('simplenews_newsletter')->loadMultiple($required_newsletters);
      $labels = [];
      foreach ($newsletters as $newsletter) {
        $labels[] = $newsletter->label();
      }

      return $this->t(
            'User subscribed to newsletters: @newsletters', [
              '@newsletters' => implode(', ', $labels),
            ]
            );
    }
  }

}
